import subprocess
import os
import re
import json
from multiprocessing import Pool


def nsenter_single_command(pid, command, timeout):
	return subprocess.run(["nsenter", "-m", "-C", "-t", str(pid), "--", *command], timeout=timeout, capture_output=True, text=True)

def nsenter_batch_command(pool_size=50, container_pid_map={}, command=[], command_timeout=5):
    result_dict = {}
    with Pool(pool_size) as p:
        jobs = {}
        for k, pid in container_pid_map.items():
            res = p.apply_async(nsenter_single_command, (pid,command,command_timeout))
            jobs[k] = res

        for k, job in jobs.items():
            resp = job.get(timeout=(command_timeout + 5))
            result_dict[k] = resp

    return result_dict

def podman_single_command(pod, command, timeout):
    return subprocess.run(["podman", "exec", pod, *command], capture_output=True, text=True)

def podman_batch_command(pool_size=50, command_timeout=10, pods=[], command=[]):
    result_dict = {}
    with Pool(pool_size) as p:
        jobs = {}
        for pod in pods:
            res = p.apply_async(podman_single_command, (pod,command, command_timeout))
            jobs[pod] = res

        for pod, job in jobs.items():
            resp = job.get(timeout=(command_timeout + 5))
            result_dict[pod] = resp

    return result_dict

def get_podman_names(pod_filter=None):
    data = get_podman_data(pod_filter)
    return [d['Names'][0] for d in data if 'Names' in d and len(d['Names'])]

def get_podman_data(pod_filter=None):
    try:
        result = subprocess.run(["podman", "ps", "--format", "json"], capture_output=True, text=True)
        data = json.loads(result.stdout)
    except Exception:
        raise Exception("failed to get pods")
    if pod_filter:
        pod_filter_re = re.compile(pod_filter)
        filtered_data = []
        for d in data:
            if 'Names' in d and len(d['Names']) > 0:
                if pod_filter_re.match(d['Names'][0]):
                    filtered_data.append(d)
        return filtered_data

    return data

def get_podman_disk_target(pod, mount):
    return os.readlink("/podman/container_data/%s/%s" % (pod, mount))
